@@ -16,7 +16,7 @@
 # Copyright 2012 Laurent Gautier
 
 import unittest
-import io
+import io, sys
 import struct
 import ngs_plumbing._libdna as _libdna
 
@@ -75,20 +75,20 @@ class BitPackingTestCase(unittest.TestCase):
         chunk = _libdna.slicebit2_frombit2bytes(packed, 0, 8)
         self.assertEqual(string[0:8], _libdna.byte_frombit2bytes(chunk))
         chunk = _libdna.slicebit2_frombit2bytes(packed, 3, 6)
-        self.assertEqual(string[3:6] + 'A'*(4-(6-3)%4), _libdna.byte_frombit2bytes(chunk))
+        self.assertEqual(string[3:6] + b'A'*(4-(6-3)%4), _libdna.byte_frombit2bytes(chunk))
         chunk = _libdna.slicebit2_frombit2bytes(packed, 2, 4)
-        self.assertEqual(string[2:4] + 'A'*(4-(4-2)%4), _libdna.byte_frombit2bytes(chunk))
+        self.assertEqual(string[2:4] + b'A'*(4-(4-2)%4), _libdna.byte_frombit2bytes(chunk))
         chunk = _libdna.slicebit2_frombit2bytes(packed, 0, 11)
-        self.assertEqual(string[0:11] + 'A'*(4-(11-0)%4), 
+        self.assertEqual(string[0:11] + b'A'*(4-(11-0)%4), 
                          _libdna.byte_frombit2bytes(chunk))
         chunk = _libdna.slicebit2_frombit2bytes(packed, 3, 11)
         self.assertEqual(string[3:11], 
                          _libdna.byte_frombit2bytes(chunk))
         chunk = _libdna.slicebit2_frombit2bytes(packed, 4, 11)
-        self.assertEqual(string[4:11] + 'A'*(4-(11-4)%4), 
+        self.assertEqual(string[4:11] + b'A'*(4-(11-4)%4), 
                          _libdna.byte_frombit2bytes(chunk))
         chunk = _libdna.slicebit2_frombit2bytes(packed, 3, 12)
-        self.assertEqual(string[3:12] + 'A'*(4-(12-3)%4),
+        self.assertEqual(string[3:12] + b'A'*(4-(12-3)%4),
                          _libdna.byte_frombit2bytes(chunk))
 
         #self.assertRaises(IndexError, _libdna.slice_frombit2bytes,
@@ -147,7 +147,14 @@ class BitPackingTestCase(unittest.TestCase):
         vector_rev = _libdna.bit2reverse_frombit2bytes(vector)
         self.assertEqual(len(vector), len(vector_rev))
         string_rev = _libdna.byte_frombit2bytes(vector_rev)
-        self.assertEqual(b''.join(string[x] for x in range(len(string)-1, -1, -1)), string_rev)
+        #Python 3:
+        if sys.version_info[0] > 2:
+            tmp = bytearray(len(string)-1)
+            for i, j in enumerate(range(len(tmp), 0, -1)):
+                tmp[i] = string[j]
+        else:
+            tmp = b''.join(string[x] for x in range(len(string)-1, -1, -1))
+            self.assertEqual(tmp, string_rev)
 
     def testBit2reverse_frombit2bytearray(self):
         string = b'ATACGCGGCT'+b'GATCGTAGCG'
@@ -155,12 +162,25 @@ class BitPackingTestCase(unittest.TestCase):
         vector_rev = _libdna.bit2reverse_frombit2bytearray(vector)
         self.assertEqual(len(vector), len(vector_rev))
         string_rev = _libdna.byte_frombit2bytearray(vector_rev)
-        self.assertEqual(b''.join(string[x] for x in range(len(string)-1, -1, -1)), string_rev)
+        #Python 3:
+        if sys.version_info[0] > 2:
+            tmp = bytearray(len(string)-1)
+            for i, j in enumerate(range(len(tmp), 0, -1)):
+                tmp[i] = string[j]
+        else:
+            tmp = b''.join(string[x] for x in range(len(string)-1, -1, -1))
+            self.assertEqual(tmp, string_rev)
 
     def testBit2reversecomplement_frombit2bytes(self):
         string =    b'ATACGCGGCT'+b'GATCGTAGCG'
         string_c = b'TATGCGCCGA'+b'CTAGCATCGC'
-        string_rc = b''.join(string_c[x] for x in range(len(string)-1, -1, -1))
+        #Python 3:
+        if sys.version_info[0] > 2:
+            string_rc = bytearray(len(string))
+            for i, j in enumerate(range(len(string_rc)-1, -1, -1)):
+                string_rc[i] = string_c[j]
+        else:
+            string_rc = b''.join(string_c[x] for x in range(len(string)-1, -1, -1))
         vector = _libdna.bit2_frombytes(string)
         vector_rc = _libdna.bit2reversecomplement_frombit2bytes(vector)
         self.assertEqual(len(vector), len(vector_rc))
@@ -170,7 +190,13 @@ class BitPackingTestCase(unittest.TestCase):
     def testBit2reversecomplement_frombit2bytearray(self):
         string =    b'ATACGCGGCT'+b'GATCGTAGCG'
         string_c = b'TATGCGCCGA'+b'CTAGCATCGC'
-        string_rc = b''.join(string_c[x] for x in range(len(string)-1, -1, -1))
+        #Python 3:
+        if sys.version_info[0] > 2:
+            string_rc = bytearray(len(string))
+            for i, j in enumerate(range(len(string_rc)-1, -1, -1)):
+                string_rc[i] = string_c[j]
+        else:
+            string_rc = b''.join(string_c[x] for x in range(len(string)-1, -1, -1))
         vector = _libdna.bit2_frombytearray(bytearray(string))
         vector_rc = _libdna.bit2reversecomplement_frombit2bytearray(vector)
         self.assertEqual(len(vector), len(vector_rc))
