@@ -19,6 +19,8 @@
 Utilities around strings
 """
 
+import numpy
+from numpy.random import random_integers
 from random import randint
 from io import IOBase
 
@@ -27,24 +29,24 @@ import sys
 if sys.version_info[0] > 2:
     long = int
     xrange = range
-    def randomstring(size, alphabet):
-        """ Return a random string of length 'size', 
-        using the given 'alphabet' of possible letters."""
-        assert(isinstance(size, long) or isinstance(size, int))
-        assert(isinstance(alphabet, bytes))
-        # make a genome
-        string = bytearray(size)
-        for i in range(size):
-            string[i] = alphabet[randint(0, 3)]
-        return string
 else:
-    def randomstring(size, alphabet):
-        """ Return a random string of length 'size', 
-        using the given 'alphabet' of possible letters."""
-        assert(isinstance(size, long) or isinstance(size, int))
-        # make a genome
-        string = b''.join(alphabet[randint(0, 3)] for x in xrange(size))
-        return string
+    pass
+
+def randomstring(size, alphabet, maxchunk=int(1000000)):
+    """ Return a random string of length 'size', 
+    using the given 'alphabet' of possible letters."""
+    assert(isinstance(size, long) or isinstance(size, int))
+    assert(isinstance(alphabet, bytes))
+    assert(isinstance(maxchunk, int))
+    # make a genome
+    string = bytearray(size)
+    chunksize = maxchunk
+    for offset in range(0, size, maxchunk):
+        if (offset+maxchunk) > size:
+            chunksize = size - offset
+        rint = random_integers(0, 3, chunksize)
+        string[offset:(offset+maxchunk)] = (alphabet[x] for x in rint)
+    return bytes(string)
 
 def randomfragments(string, n, size = 100):
     """ Make 'n' random, possibly overlapping, fragments
