@@ -29,19 +29,22 @@ bitwise_and = numpy.bitwise_and
 from ngs_plumbing.dnaqual import DnaQual
 from ngs_plumbing._libxsq import colourqual_frombuffer, basequal_frombuffer, bytearray_phredtoascii, bytearray_addint
 
+if sys.version_info[0] > 2:
+    xrange = range
+
 ColourQual = namedtuple('ColourQual', 'colour qual')
 
-_ACGT = bytes('ACGT')
+_ACGT = 'ACGT'
 FRAGMENT_NAMES = ('F3', 'F5-DNA', 'F5-RNA', 'F5-BC', 'F5-2', 'R3', 'R5')
 # F3: Forward strand. starts with a T
 # F5/R3: Used to prime both F5-BC read and R3 or barcode. starts with a G
 #
-FRAGMENT_START = {'F3': b'T',
-                  'F5-DNA': b'G',
-                  'F5-RNA': b'G', # to be verified
+FRAGMENT_START = {'F3': 'T',
+                  'F5-DNA': 'G',
+                  'F5-RNA': 'G', # to be verified
                   'F5-BC': None,
                   'F5-2': None,
-                  'R3': b'G',
+                  'R3': 'G',
                   'R5': None # I don't know
 }
 
@@ -127,7 +130,7 @@ class XSQLibrary(h5py._hl.group.Group):
     def readcount(self):
         """ Return the number of reads for the library
         """
-        count = long(0)
+        count = int(0)
         for number in self.keys():
             tmp = super(XSQLibrary, self).__getitem__(number)
             #FIXME: hack !
@@ -216,11 +219,17 @@ def iter_valuearrays(group, fragment, what='BaseCallQV'):
     """ Iterator over the fragment 'fragment' for the data 'what'
     """
     assert(fragment in FRAGMENT_NAMES)
-    for tile_n in group.keys():
-        tile = group[tile_n]
-        a = tile_valuearray(tile, fragment, what)
-        tile_n = bytes(tile_n)
-        yield (tile_n, a)
+    if sys.version_info[0] > 2:
+        for tile_n in group.keys():
+            tile = group[tile_n]
+            a = tile_valuearray(tile, fragment, what)
+            tile_n = bytes(tile_n, 'ASCII')
+            yield (tile_n, a)
+    else:
+        for tile_n in group.keys():
+            tile = group[tile_n]
+            a = tile_valuearray(tile, fragment, what)
+            yield (tile_n, a)
 
 def iter_reads(group, fragment, what='BaseCallQV'):
     """ Iterator over the fragment 'fragment' for the data 'what'
@@ -370,8 +379,7 @@ def seqid_template(f, flowcell, tile):
     try:
         serial = f.run_metadata.attrs['InstrumentSerial'][0]
     except (AttributeError, KeyError) as ke:
-        serial = 'UnknownSerial'
-        
+        serial = b'UnknownSerial'
     seq_id = '@%s:%s:%s' %(serial, flowcell, tile) + ':%i:%i#0'
     return seq_id
 
@@ -381,21 +389,25 @@ def seqid_template_csfasta(flowcell, fragment):
 
 def _progress_bar_iter(iterator, size = 20, out = sys.stdout, suffix = ''):
     n = len(iterator)
-    ms = long(math.ceil(1.0 * n / size))
+    ms = int(math.ceil(1.0 * n / size))
+    if sys.version_info[0] == 2:
+        block = '\xe2\x96\x88'
+    else:
+        block = u"\u2588"
     for j, i in enumerate(iterator):
         if j % ms == 0:
             if os.isatty(out.fileno()):
-                bar = '\r|%s%s|%s' %('\xe2\x96\x88' * min(size, (j / ms)), ' ' * (size - (j / ms)), suffix)
+                bar = '\r|%s%s|%s' %(block * min(size, (j / ms)), ' ' * (size - (j / ms)), suffix)
                 out.write(bar)
                 out.flush()
         yield i
 
 def make_fastq(lib, fragment, buf, f, flowcell, numbase = None, 
                progress_mark = 50000, out=sys.stdout):
-    n_count = long(0)
-    n_perfect = long(0)
-    read_count = long(0)
-    inc = long(1)
+    n_count = int(0)
+    n_perfect = int(0)
+    read_count = int(0)
+    inc = int(1)
 
     for tile_i, (tile_n, a) in enumerate(iter_valuearrays(lib, fragment, what='BaseCallQV')):
         if numbase is None:
@@ -406,8 +418,8 @@ def make_fastq(lib, fragment, buf, f, flowcell, numbase = None,
 
         read_count += xyloc.shape[0]
                 
-        ms = long(math.ceil((tile_i+1) * 1.0 * 20 / len(lib)))
-        out.write('\r|%s%s|tile %i' %('\xe2\x96\x88' * min(20, ms), 
+        ms = int(math.ceil((tile_i+1) * 1.0 * 20 / len(lib)))
+        out.write('\r|%s%s|tile %i' %(block * min(20, ms), 
                                       ' ' * (20 - ms), tile_i+1))
         out.flush()
 
@@ -420,7 +432,7 @@ def make_fastq(lib, fragment, buf, f, flowcell, numbase = None,
             n_count += n_count_read
             if n_count_read == 0:
                 n_perfect += inc
-            dna_b = bytes(dna)
+            dna_b = bytes(dna, 'ASCII')
             x,y = xyloc[row_i]
             buf.writelines((seq_id %(x,y), '\n', dna_b, '\n+\n', qual, '\n'))
 
@@ -441,21 +453,38 @@ def _xsqvalues_to_csfasta(xsqvalues, xyloc, buf_cs, buf_qual, seq_id, prefix):
         sys.stderr.flush()
         return
     bases_i = range(xsqvalues.shape[1]-1)
-    for row_i in xrange(xsqvalues.shape[0]): 
-        seq, qual = colourqual_frombuffer(xsqvalues[row_i])
-        x,y = xyloc[row_i]
-        this_seq_id = seq_id % (x,y)
-        buf_cs.writelines((this_seq_id, '\n', prefix, seq, '\n'))
-        buf_qual.writelines((this_seq_id, '\n'))
-        for q_i in bases_i:
-            buf_qual.write('%i ' % qual[q_i])
-        buf_qual.writelines((str(qual[len(bases_i)]), '\n'))
+    if sys.version_info[0] > 2:
+        for row_i in xrange(xsqvalues.shape[0]): 
+            seq, qual = colourqual_frombuffer(xsqvalues[row_i])
+            x,y = xyloc[row_i]
+            this_seq_id = seq_id % (x,y)
+            buf_cs.writelines((this_seq_id, '\n', 
+                               prefix, seq.decode('ASCII'), '\n'))
+            buf_qual.writelines((this_seq_id, '\n'))
+            for q_i in bases_i:
+                buf_qual.write('%i ' % qual[q_i])
+            buf_qual.writelines((str(qual[len(bases_i)]), '\n'))
+    else:
+        for row_i in xrange(xsqvalues.shape[0]): 
+            seq, qual = colourqual_frombuffer(xsqvalues[row_i])
+            x,y = xyloc[row_i]
+            this_seq_id = seq_id % (x,y)
+            buf_cs.writelines((this_seq_id, '\n', 
+                               prefix, seq, '\n'))
+            buf_qual.writelines((this_seq_id, '\n'))
+            for q_i in bases_i:
+                buf_qual.write('%i ' % qual[q_i])
+            buf_qual.writelines((str(qual[len(bases_i)]), '\n'))
 
 
 def make_csfasta(lib, fragment, buf_cs, buf_qual, f, flowcell, 
                  numbase = None, progress_mark = 50000, out=sys.stdout):
-    read_count = long(0)
-    inc = long(1)
+    read_count = int(0)
+    inc = int(1)
+    if sys.version_info[0] == 2:
+        block = '\xe2\x96\x88'
+    else:
+        block = u"\u2588"
 
     for tile_i, (tile_n, a) in enumerate(iter_valuearrays(lib, fragment, 
                                                           what='ColorCallQV')):
@@ -466,8 +495,8 @@ def make_csfasta(lib, fragment, buf_cs, buf_qual, f, flowcell,
         read_count += xyloc.shape[0]
         prefix = FRAGMENT_START[fragment]
 
-        ms = long(math.ceil((tile_i + 1) * 1.0 * 20 / len(lib)))
-        out.write('\r|%s%s|tile %i' %('\xe2\x96\x88' * min(20, ms), 
+        ms = int(math.ceil((tile_i + 1) * 1.0 * 20 / len(lib)))
+        out.write('\r|%s%s|tile %i' %(block * min(20, ms), 
                                       ' ' * (20 - ms), tile_i + 1))
         out.flush()
 
@@ -479,17 +508,32 @@ def make_csfasta(lib, fragment, buf_cs, buf_qual, f, flowcell,
     out.flush()
 
 def _xsqvalues_to_csfastq(xsqvalues, xyloc, buf, seq_id, prefix):
-    for row_i in xrange(xsqvalues.shape[0]):
-        seq, qual = colourqual_frombuffer(xsqvalues[row_i])
-        seq = bytearray_addint(seq, 48)
-        qual = bytearray_phredtoascii(qual)
-        x,y = xyloc[row_i]
-        buf.writelines((seq_id %(x,y), '\n', prefix, seq, '\n+\n', qual, '\n'))
+    if sys.version_info[0] > 2:
+        for row_i in xrange(xsqvalues.shape[0]):
+            seq, qual = colourqual_frombuffer(xsqvalues[row_i])
+            seq = bytearray_addint(seq, 48)
+            qual = bytearray_phredtoascii(qual)
+            x,y = xyloc[row_i]
+            buf.writelines((seq_id %(x,y), '\n', prefix, seq.decode('ASCII'), 
+                            '\n+\n', qual.decode('ASCII'), '\n'))
+    else:
+        seq_id = seq_id.encode('ASCII')
+        for row_i in xrange(xsqvalues.shape[0]):
+            seq, qual = colourqual_frombuffer(xsqvalues[row_i])
+            seq = bytearray_addint(seq, 48)
+            qual = bytearray_phredtoascii(qual)
+            x,y = xyloc[row_i]
+            buf.writelines((seq_id %(x,y), '\n', prefix, seq, 
+                            '\n+\n', qual, '\n'))
         
 def make_csfastq(lib, fragment, buf, f, flowcell, 
                  numbase = None, progress_mark = 50000, out=sys.stdout):
-    read_count = long(0)
-    inc = long(1)
+    read_count = int(0)
+    inc = int(1)
+    if sys.version_info[0] == 2:
+        block = '\xe2\x96\x88'
+    else:
+        block = u"\u2588"
 
     for tile_i, (tile_n, a) in enumerate(iter_valuearrays(lib, fragment, 
                                                           what='ColorCallQV')):
@@ -500,8 +544,8 @@ def make_csfastq(lib, fragment, buf, f, flowcell,
         read_count += xyloc.shape[0]
         prefix = FRAGMENT_START[fragment]
 
-        ms = long(math.ceil((tile_i+1) * 1.0 * 20 / len(lib)))
-        out.write('\r|%s%s|tile %i' %('\xe2\x96\x88' * min(20, ms), 
+        ms = int(math.ceil((tile_i+1) * 1.0 * 20 / len(lib)))
+        out.write('\r|%s%s|tile %i' %(block * min(20, ms), 
                                       ' ' * (20 - ms), tile_i + 1))
         out.flush()
 
@@ -514,7 +558,7 @@ def make_csfastq(lib, fragment, buf, f, flowcell,
 
 
 def automagic_fastq(filename, path_out = '.', fragment = None, 
-                    buf_size = long(2E6), extension = 'fq', out = sys.stdout,
+                    buf_size = int(2E6), extension = 'fq', out = sys.stdout,
                     cs = False,
                     want_unassigned = False):
     """ Extract sequence-space data into a FASTAQ file """
@@ -555,7 +599,7 @@ def automagic_fastq(filename, path_out = '.', fragment = None,
     f.close()
 
 def automagic_csfasta(filename, path_out = '.', fragment = None, 
-                      buf_size = long(2E6), out = sys.stdout,
+                      buf_size = int(2E6), out = sys.stdout,
                       want_unassigned = False):
     """ Extract sequence-space data into a FASTAQ file """
 
@@ -612,7 +656,7 @@ def fastqual(xsqlib, fragment, what, sample = 0.05):
 
     # determine the indices for the reads sampled
     nfragments = xsqlib.readcount()
-    nsample = long(sample * nfragments)
+    nsample = int(sample * nfragments)
     reads_idx = [None, ] * nsample
     for i in xrange(nsample):
         reads_idx[i] = random.randint(0, nfragments-1)
